package conversion;

import java.util.Vector;

import structuresgraphique.*;
import structuresdonnees.*;

public class Conversion
{
	/**
	 * Conversion de la structure de donnes  la
	 * structure graphique
	 * @param la structure de donnes
	 * @return la structure graphique correspondante
	 */
	public static UnArbreGraphique donneeToGraphic(ABP telArbreDonnees)
	{
		UnArbreGraphique telArbre=donneeToGraphic(telArbreDonnees, 1);
		telArbreDonnees.calculStrahler (1);
		if(telArbre != null)
			telArbre.maj_x_y();
		
		return telArbre;
	}

	/**
	 * Conversion de la structure de donnes  la
	 * structure graphique
	 * @param la structure de donnes
	 * @return l'indice du noeud  traiter
	 */	
	private static UnArbreGraphique donneeToGraphic(ABP telArbreDonnees, int telInd)
	{
		// Creation d'un arbre avec seulement une racine
		UnArbreGraphique lArbreTemp = new UnArbreGraphique();
		
		// Test si le noeud existe
		if(telArbreDonnees.getObjNoeud(telInd) != null)
		{
			// Stocke le nombre de shraler
			lArbreTemp.getSonSommet().setStrahler(telArbreDonnees.getObjNoeud(telInd).getStrahler());
			
			// Stocke la couleur
			lArbreTemp.getSonSommet().setSaCouleur(telArbreDonnees.getObjNoeud(telInd).getCouleur());
			
			// Stocke le reste des donnes dans le sommet
			lArbreTemp.getSonSommet().addData(telArbreDonnees.getObjNoeud(telInd).getInfosNoeud());
			
			// Test si les fils du noeud existent (arbre parfait)
			if(telArbreDonnees.getObjNoeud(2*telInd) != null)
			{
				// Cree les fils
				lArbreTemp.addFils(donneeToGraphic(telArbreDonnees,2*telInd),donneeToGraphic(telArbreDonnees,2*telInd+1));
				
				// Stocke les infos dans les aretes si besoin
				lArbreTemp.setSesDonneesAreteGauche(convertVectorToListe(telArbreDonnees.getObjNoeud(2*telInd).getInfosArete()));
				lArbreTemp.setSesDonneesAreteDroite(convertVectorToListe(telArbreDonnees.getObjNoeud(2*telInd+1).getInfosArete()));
				
			}	
		}
		else
			return null;
	
		return lArbreTemp;
	}

	/**
	 * Conversion de la structure de donnes  la
	 * structure graphique
	 * @param la structure de donnes
	 * @return la structure graphique correspondante
	 */	
	public static ABP graphicToDonnees(UnArbreGraphique telArbreGraphique)
	{
		ABP telArbre = new ABP(telArbreGraphique.getSaHauteur());
		
		Noeud[] leTabNoeud = profondeur(telArbreGraphique, null,1);
		
		telArbre.setArbre(leTabNoeud);
		
				
		return telArbre;
	}
	
	/**
	 * Rempli le tableau pour convertir de la structure graphique
	 * et la structure de donnes
	 * @param telArbreGraphique l'arbre graphique
	 * @param lesSommets le tableau de sommets (graphiques)
	 * @param ind indice  remplir dans le tableau
	 * @retourne tableau de noeuds (structure de donnes)
	 */
	private static Noeud[] profondeur(UnArbreGraphique telArbreGraphique, UnSommet[] lesSommets, int ind)
	{
		int hauteur = telArbreGraphique.getSaHauteur();
		
		if(lesSommets==null)
			lesSommets=new UnSommet[(int)Math.pow(2,telArbreGraphique.getSaHauteur())];
			
		
		if(telArbreGraphique.getSonSommet()!=null)
		{
			lesSommets[ind]=telArbreGraphique.getSonSommet();
			
			if(telArbreGraphique.getSonFilsGauche()!=null)
			{
				profondeur(telArbreGraphique.getSonFilsGauche(), lesSommets, 2*ind);
				profondeur(telArbreGraphique.getSonFilsDroit(), lesSommets, 2*ind+1);
			}
			
			if(ind == 1)
			{
				// Lance la conversion
				
				Noeud[] leTabNoeud = new Noeud[(int)Math.pow(2,telArbreGraphique.getSaHauteur())];
				
				if(ind ==1)
				for(int i = 1; i<lesSommets.length;i++)
				{
					// On recopie les proprits du sommet
					if(lesSommets[i]!=null)
					{
						leTabNoeud[i]=new Noeud();
						leTabNoeud[i].setCouleur(lesSommets[i].getSaCouleur());
						leTabNoeud[i].setInfosNoeud(convertListToVector(lesSommets[i].getSesDonnees(), true));
					}
					else 
						leTabNoeud[i]=null;
				}
				
				for(int i = 1; i<lesSommets.length;i++)
				{
					// Copie des donnes de l'arete allant vers son pre
					if(i*2<lesSommets.length && leTabNoeud[i*2]!=null)
						leTabNoeud[i*2].setInfosArete(convertListToVector(telArbreGraphique.trouveSommetArbre(lesSommets[i]).getSesDonneesAreteGauche(),false));
						
					if(i*2+1<lesSommets.length && leTabNoeud[i*2+1]!=null)
						leTabNoeud[i*2+1].setInfosArete(convertListToVector(telArbreGraphique.trouveSommetArbre(lesSommets[i]).getSesDonneesAreteDroite(),false));
										
				}
				
				return leTabNoeud;
			}	
			
		}
		return null;
	}
	
	/**
	 * Retourne le vecteur contenant les donnes
	 * @return Vector le vecteur des donnes
	 */
	public static Vector convertListToVector(ListeStockage telleListe, boolean zapFirst)
	{
		if(telleListe==null || telleListe.size()==0)
			return null;
		
		Vector leVecteur=new Vector();
		
		int i;
		
		i= (zapFirst) ? 1 : 0; 
		
		for(; i<telleListe.size();i++)
		{
			String [] leTabTemp=new String[2];
			Stockage leStockageTemp=telleListe.getStockage(i);
			
			leTabTemp[0]=new String(leStockageTemp.getNom());
			leTabTemp[1]=new String(leStockageTemp.getContenu());
			
			leVecteur.add(leTabTemp);		
		}
		return leVecteur;
	}

	/**
	 * Ajoute les donnes  la liste de stockage
	 * @param tellesDonnees le vecteur contenant les donns  copier
	 */
	public static ListeStockage convertVectorToListe(Vector tellesDonnees)
	{
		ListeStockage laListe=new ListeStockage();
			
		if(tellesDonnees !=null && tellesDonnees.size()>=0)
		{
			for(int i=0;i<tellesDonnees.size();i++)
			{	
				String [] laLigne=(String[])tellesDonnees.get(i);
				laListe.addStockage(new Stockage(new String(laLigne[0]),new String(laLigne[1])));
			}
		}
		return laListe;
	}
	
}
