package editeurarbre;

import javax.swing.table.AbstractTableModel;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import javax.swing.JCheckBox;
import java.util.Vector;
import editeurarbre.Constantes;
import structuresgraphique.Stockage;
import structuresgraphique.ListeStockage;


class TableauArbreModel extends AbstractTableModel implements TableModelListener
{
	private ListeStockage sonContenu=null;
	private Vector sesBoolSuppr=null;
	private ChangementColSupprListener sonCCSL=null;
	private boolean sonBoolPremierIsRemovable;
	
	public TableauArbreModel(ListeStockage telContenu, boolean telBoolPremierIsRemovable) throws NullPointerException
	{
		if(telContenu==null)
			throw new NullPointerException("Le contenu du tableau ne doit pas tre null");
			
		sonBoolPremierIsRemovable=telBoolPremierIsRemovable;
		
		addTableModelListener(this);
		
		sonContenu=telContenu;
		sesBoolSuppr=new Vector();
		
		int leNb=sonContenu.size();
		
		for(int i=0; i<leNb; i++)
			sesBoolSuppr.addElement(Boolean.FALSE);
		
	}
	
	public int getRowCount()
	{
		if(sonContenu!=null)
			return sonContenu.size();
		else
			return 0;
	}
	
	public int getColumnCount()
	{
		return Constantes.NOM_COLS_TABLEAU.length;
	}
	
	public Object getValueAt(int telleL, int telleC)
	{
		switch(telleC)
		{
			case Constantes.SUPPR:
				return (Boolean)sesBoolSuppr.get(telleL);
				
			case Constantes.NOM:
				return sonContenu.getStockage(telleL).getNom();
			
			case Constantes.CONTENU:
				return sonContenu.getStockage(telleL).getContenu();
		}
		
		return null;
	}
	
	public void setValueAt(Object telObj, int telleL, int telleC)
	{
		switch(telleC)
		{
			case Constantes.SUPPR:
				sesBoolSuppr.set(telleL, telObj);
				break;
				
			case Constantes.NOM:
				((Stockage)sonContenu.getStockage(telleL)).setNom((String)telObj);
				break;
			
			case Constantes.CONTENU:
				((Stockage)sonContenu.getStockage(telleL)).setContenu((String)telObj);
				break;
		}
		
		//si la colonne a mettre a jour est la col suppr
		//on envoi l evenement
		//if(telleC==Constantes.SUPPR)
		TableModelEvent leTME=new TableModelEvent(this, Constantes.SUPPR, Constantes.CONTENU, telleC, 0);
		fireTableChanged(leTME);
	}
	
	public Class getColumnClass(int telleC)
	{
		return getValueAt(0, telleC).getClass();
	}
	
	public boolean isCellEditable(int telleL, int telleC)
	{
		if(!premierEltIsRemovable() && telleL==0)
			return false;
			
		if(sonContenu.size()<=Constantes.NB_ELT_MIN && telleC==Constantes.SUPPR)
			return false;
		else
			return true;
	}
	
	public String getColumnName(int telleC)
	{
		return Constantes.NOM_COLS_TABLEAU[telleC];
	}
	
	public void setContenu(ListeStockage telContenu)
	{
		sonContenu=telContenu;
	}
	
	public void ajouteEltTableau(Stockage telElt)
	{
		sonContenu.addStockage(telElt);
		sesBoolSuppr.addElement(Boolean.FALSE);
	}
	
	public boolean retireEltTableau(Stockage telElt)
	{
		boolean ok=false;
		int i;
		
		int leNb=sonContenu.size();
		for(i=0; i<leNb && !ok; i++)
		{
			if(sonContenu.getStockage(i)==telElt)
				ok=true;
		}
		
		//on ne peut pas enlever le premier element
		if(!sonBoolPremierIsRemovable && i-1==0)
			return false;
		
		if(ok && sonContenu!=null && sonContenu.size()>0)
		{
			sonContenu.removeStockage(i-1);
			sesBoolSuppr.removeElementAt(i-1);
		}
		else
		{
			ok=false;
		}
		
		updateStatusBool();
		
		return ok;
	}
	
	public boolean retireEltTableau(int telRang)
	{
		boolean ok=true;
		
		//on ne peut pas enlever le premier element
		if(!sonBoolPremierIsRemovable && telRang==0)
			return false;
		
		if(sonContenu!=null && sonContenu.size()>0)
		{
			sonContenu.removeStockage(telRang);
			sesBoolSuppr.removeElementAt(telRang);
		}
		else
		{
			ok=false;
		}
		
		updateStatusBool();
		
		return ok;
	}
	
	private void updateStatusBool()
	{
		if(sonContenu.size()<=Constantes.NB_ELT_MIN)
			for(int i=0; i<Constantes.NB_ELT_MIN; i++)
				setValueAt(Boolean.FALSE, i, Constantes.SUPPR);
	}
	
	public int getNbLigne()
	{
		return sonContenu.size();
	}
	
	public boolean isSelected(int telleL)
	{
		return ((Boolean)getValueAt(telleL, Constantes.SUPPR)).booleanValue();
	}
	
	public void selectAll()
	{
		int leNb=this.sesBoolSuppr.size();
		
		int leDebut=0;
		if(!premierEltIsRemovable())
			leDebut++;
			
		for(int i=leDebut; i<leNb; i++)
			setValueAt(Boolean.TRUE, i, Constantes.SUPPR);
	}
	
	public void deselectAll()
	{
		int leNb=this.sesBoolSuppr.size();
		
		for(int i=0; i<leNb; i++)
			setValueAt(Boolean.FALSE, i, Constantes.SUPPR);
	}
	
	public void revalidate()
	{
		//pour revalider, on a besoin de s assurer que le tableau de booleens
		//est de la meme taille que la liste stockage
		int leNbContenu=sonContenu.size();
		int leNbBool=sesBoolSuppr.size();
		
		if(leNbContenu>leNbBool)
		{
			for(int i=leNbBool; i<leNbContenu; i++)
				sesBoolSuppr.addElement(Boolean.FALSE);
		}
		else
		{
			if(leNbContenu<leNbBool)
			{
				for(int i=leNbBool; i<leNbContenu; i++)
					sesBoolSuppr.remove(sesBoolSuppr.size()-1);
			}
		}
	}
	
	public void tableChanged(TableModelEvent e)
	{
		if(sonCCSL!=null)
			sonCCSL.changementColSupprTable(e.getColumn());
	}
	
	public void addChangementColSupprListener(ChangementColSupprListener telCCSL)
	{
		sonCCSL=telCCSL;
	}
	
	public boolean aucuneCaseSupprSelectionnee()
	{
		boolean ok=true;
		int leNb=sesBoolSuppr.size();
		
		for(int i=0; i<leNb && ok; i++)
		{
			if(((Boolean)sesBoolSuppr.get(i)).booleanValue())
				ok=false;
		}
		
		return ok;
	}
	
	public void synchronizeBoolSupprAvecContenu()
	{
		sesBoolSuppr.removeAllElements();
		int leNb=sonContenu.size();
		
		for(int i=0; i<leNb; i++)
			sesBoolSuppr.addElement(Boolean.FALSE);
	}
	
	public boolean premierEltIsRemovable()
	{
		return sonBoolPremierIsRemovable;
	}
	
	/*public void afficheContenu()
	{
		ListeStockage.afficheListeStockage(sonContenu);
	}*/
}