package modulecreation;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;

import structuresgraphique.UnSommet;
import structuresgraphique.UnArbreGraphique;

import editeurarbre.ihmediteurarbre.FenetreEditeurArbre;

/**
 * Cette classe reprsente un panel de dessin dans lequel
 * on peut facilement crer des arbres  l'aide la souris
 * Celle ci hrite de la class UnPanelArbre qui permet
 * juste de les afficher
 * @author Matthieu LOMON - Nicolas Maupu
 */
public class UnPanelDessin extends UnPanelArbre implements MouseMotionListener, MouseListener, ActionListener
{
	// Pour bouger une forme
	private boolean sonBoolMove = false ;
	
	// Le menu contextuel
	private JPopupMenu sonMenuPopup = new JPopupMenu();
	private String sesIntitulesMenu[] ={
		ConstantesCreation.JMENU_SUPPRIMER,
		ConstantesCreation.JMENU_MODIFIER,
		ConstantesCreation.JMENU_MODIFIERG, 
		ConstantesCreation.JMENU_MODIFIERD,
		ConstantesCreation.JMENU_NOUVEAU,
		ConstantesCreation.JMENU_DEP_SS_A,
		ConstantesCreation.JMENU_COP_SS_A,
		ConstantesCreation.JMENU_ANNULER};
	private JMenuItem sesItemsPopup[]= new JMenuItem[sesIntitulesMenu.length];
	
	// Le sommet trouv quand on clique dessus
	private UnSommet sonSommetTrouve = null;
	
	// L'arbre dont les fils sont  dplacer
	private static UnArbreGraphique sonArbreADeplacer = null;
	
	// L'arbre dont les fils sont  dplacer
	private static UnArbreGraphique sonArbreACopier = null;

	// Est vrai lorsqu'on cre un fils
	private boolean sonBoolCreateFils = false;
	
	// Le panel qui contient ce panel
	private UnPanelDessinBase sonPanelDessinAll = null;
	
	// Sauvegarde de l'arbre
	private UnArbreGraphique saCopieArbre = null;
	
	//
	private static boolean sonFirstMsgDep = false;
	private static boolean sonFirstMsgCop = false;
	
	/**
	 * Constructeur
	 * @param telArbre l'arbre  afficher dans le panel
	 */
	public UnPanelDessin(UnArbreGraphique telArbre, UnPanelDessinBase telPanelDessinAll)
	{
		super(telArbre);
		
		sonPanelDessinAll = telPanelDessinAll;
		
		createJPopupMenu();

		addMouseMotionListener(this);
		addMouseListener(this);	
		setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
		setPreferredSize(new Dimension(ConstantesCreation.LARGEUR_PANEL, ConstantesCreation.HAUTEUR_PANEL));
		
	}
	
	/**
	 * Cree le menu contextuel des sommets
	 */
	private void createJPopupMenu()
	{
		for(int i = 0; i<sesItemsPopup.length ; i++)
		{
			sesItemsPopup[i]=new JMenuItem(sesIntitulesMenu[i]);
			sesItemsPopup[i].addActionListener(this);
			sonMenuPopup.add(sesItemsPopup[i]);
			
			if(i==0 || i==3 || i==6)
				sonMenuPopup.addSeparator();	
		}
		sonMenuPopup.addMouseListener(this);		
	}
	
	/**
	 * Evnements lis aux items du popup
	 * @param e l'vnement correspondant
	 */
	public void actionPerformed(ActionEvent e)
	{
		Object obj = e.getSource();
		
		// Suppression d'un sommet
		if(obj == sesItemsPopup[0])
		{
			if(sonSommetTrouve != sonArbre.getSonSommet())
			{
				// Copie de l'arbre pour annulation 
				saCopieArbre = sonArbre.copyArbre();
				
				sonArbre.supprimerSommet(sonSommetTrouve);
				sonArbre.maj_x_y();
				repaint();
			}
			return;
		}
		
		// Nouvel arbre
		if(obj == sesItemsPopup[4])
		{
			UnArbreGraphique laCopie = sonArbre.trouveSommetArbre(sonSommetTrouve).copyArbre();
			laCopie.maj_x_y();
			laCopie.getSonSommet().setSonId(1);
			
			sonPanelDessinAll.addNewPanel(laCopie);	
			return;			
		}
		
		// Dplacement d'un sous arbre		
		if(obj == sesItemsPopup[5])
		{
			sonArbreACopier = null;
			if(!sonFirstMsgDep)
			{
				JOptionPane.showMessageDialog(this.getParent(), "<HTML>Cliquer sur le sommet sur lequel vous voulez deplacer ce sous arbre<BR>(Ce message ne sera plus affich)</HTML>");
				sonFirstMsgDep=true;
			}
			sonArbreADeplacer=sonArbre.trouveSommetArbre(sonSommetTrouve);
			return;
		}
		
		// Copie d'un sous arbre		
		if(obj == sesItemsPopup[6])
		{
			sonArbreADeplacer = null;
			if(!sonFirstMsgCop)
			{
				JOptionPane.showMessageDialog(this.getParent(), "<HTML>Cliquer sur le sommet sur lequel vous voulez copier ce sous arbre<BR>(Ce message ne sera plus affich)</HTML>");
				sonFirstMsgCop=true;
			}
			sonArbreACopier=sonArbre.trouveSommetArbre(sonSommetTrouve);
			return;
		}
		
		// Modifier les donnes (sommet)
		if(obj == sesItemsPopup[1])
		{
			saCopieArbre=null;
			new FenetreEditeurArbre(sonSommetTrouve.getSesDonnees(), "Modification des donnes du sommet " + sonSommetTrouve.getSonId() , false);
			return;
		}
		
		// Modifier les donnes (arete gauche)
		if(obj == sesItemsPopup[2])
		{
			saCopieArbre=null;
			new FenetreEditeurArbre(sonArbre.trouveSommetArbre(sonSommetTrouve).getSesDonneesAreteGauche(), "Modification des donnes de l'arte gauche du sommet " + sonSommetTrouve.getSonId(), true);
			return;
		}
		
		// Modifier les donnes (arete droite)
		if(obj == sesItemsPopup[3])
		{
			saCopieArbre=null;
			new FenetreEditeurArbre(sonArbre.trouveSommetArbre(sonSommetTrouve).getSesDonneesAreteDroite(), "Modification des donnes de l'arte droite du sommet " + sonSommetTrouve.getSonId(), true);
			return;
		}				
		
	}
	
	/**
	 * Dplacement bouton cliqu (drag)
	 * @param e l'vnement correspondant
	 */
	public void mouseDragged(MouseEvent e)
	{	
		if(sonBoolMove)
		{	
			try
			{
				if(!bougeSommet(e.getX(),e.getY(), sonSommetTrouve))
					e.consume();	
				Thread.sleep(30);				
			}
			catch(InterruptedException ex)
			{}
		}
	}
	
	/**
	 * Quand on relache le bouton de la souris (aprs un pressed
	 * @param e l'vnement correspondant
	 */
	public void mouseReleased(MouseEvent e)
	{
		// Fin du dplacement
		if(sonBoolMove)
		{
			sonBoolMove=false;
			sonArbre.maj_x_y();
			
			if(saCopieArbre!=null)
			{
				saCopieArbre.getSonSommet().setPosition(sonArbre.getSonSommet().getX(),sonArbre.getSonSommet().getY());
				saCopieArbre.maj_x_y();
			}
				
		}

		affichePopup(e);
	}
	
	public void affichePopup(MouseEvent e)
	{
		// Menu contextuel
		if(e.isPopupTrigger())
		{
			if((sonSommetTrouve=detecteSommet(e.getX(), e.getY()))!=null)
			{	
				sonBoolMove=false;
				
				// Active ou non certaines option du menu contextuel
				if(sonSommetTrouve == sonArbre.getSonSommet())
				{
					sesItemsPopup[0].setEnabled(false);
					
					if(sonArbre.getSonFilsGauche() != null)
					{
						sesItemsPopup[2].setEnabled(true);
						sesItemsPopup[3].setEnabled(true);
						sesItemsPopup[6].setEnabled(true);
					}
					else
					{
						sesItemsPopup[2].setEnabled(false);
						sesItemsPopup[3].setEnabled(false);
						sesItemsPopup[6].setEnabled(false);						
					}
				//	sesItemsPopup[4].setEnabled(false);
					sesItemsPopup[5].setEnabled(false);	
				}
				else
				{
					sesItemsPopup[0].setEnabled(true);
					
					if(sonArbre.trouveSommetArbre(sonSommetTrouve).estFeuille())
					{
						sesItemsPopup[5].setEnabled(false);
						sesItemsPopup[6].setEnabled(false);
						sesItemsPopup[2].setEnabled(false);
						sesItemsPopup[3].setEnabled(false);
						sesItemsPopup[4].setEnabled(false);
					}
					else
					{
						sesItemsPopup[5].setEnabled(true);
						sesItemsPopup[6].setEnabled(true);	
						sesItemsPopup[2].setEnabled(true);
						sesItemsPopup[3].setEnabled(true);
						sesItemsPopup[4].setEnabled(true);
					}
				}
				sonMenuPopup.show((Component)e.getSource(), (int)e.getX(), (int)e.getY());
			}
		}
	}

	/**
	 * On presse un bouton de la souris (sans le relacher)
	 * @param e l'vnement correspondant
	 */
	public void mousePressed(MouseEvent e)
	{
		// Deplacement d'objets : bouton gauche
		if(SwingUtilities.isLeftMouseButton(e))
		{
			if((sonSommetTrouve=detecteSommet(e.getX(), e.getY()))!=null)
				sonBoolMove=true;
			else
			{
				sonArbreADeplacer=null;	
				sonArbreACopier=null;	
			}
		}
		
		affichePopup(e);
	}

	/**
	 * On clique sur un bouton de la souris
	 * @param e l'vnement correspondant
	 */	
	public void mouseClicked(MouseEvent e)
	{
		// Creation d'un cercle (clic du bouton gauche)
		if(SwingUtilities.isLeftMouseButton(e))
		{
			// Deplacement d'un sous-arbre
			if(sonArbreADeplacer !=null)
			{	
				// Copie de l'arbre pour annulation 
				saCopieArbre = sonArbre.copyArbre();		
				
				UnArbreGraphique telArbre = sonArbre.trouveSommetArbre(sonSommetTrouve);
				
				// Vrifie que l'arbre peut accepter des fils
				if(telArbre.estVide())
				{
					// Verifie que le sommet ne fait pas partie du meme arbre
					if(sonArbreADeplacer.trouveSommetArbre(sonSommetTrouve) == null)
					{
						telArbre.addFils(sonArbreADeplacer.getSonFilsGauche(), sonArbreADeplacer.getSonFilsDroit(), sonArbreADeplacer);
						sonArbreADeplacer.supprimeFils();
						sonArbre.maj_x_y();
						repaint();
				
						sonSommetTrouve = null;
						sonArbreADeplacer = null;
					}
					else
						JOptionPane.showMessageDialog(this.getParent(),"Ce sommet fait partie du sous-arbre  dplacer.\nChoisissez un autre sommet");	
				}
				else
					JOptionPane.showMessageDialog(this.getParent(),"Ce sommet a dj des fils.\nChoisissez un autre sommet");	

				return;		
			}
			
			// Copie d'un sous-arbre
			if(sonArbreACopier!=null)
			{	
				// Copie de l'arbre pour annulation 
				saCopieArbre = sonArbre.copyArbre();		
				
				UnArbreGraphique telArbre = sonArbre.trouveSommetArbre(sonSommetTrouve);

				// Vrifie que l'arbre peut accepter des fils
				if(telArbre.estVide())
				{
					UnArbreGraphique telArbreCopie=sonArbreACopier.copyArbre();

					// Verifie que le sommet ne fait pas partie du meme arbre
					if(telArbreCopie.trouveSommetArbre(sonSommetTrouve) == null)
					{	
						telArbre.addFils(telArbreCopie.getSonFilsGauche(), telArbreCopie.getSonFilsDroit(), telArbreCopie);
	
						sonArbre.maj_x_y();
						repaint();
				
						sonSommetTrouve = null;
						sonArbreACopier = null;
					}
					else
						JOptionPane.showMessageDialog(this.getParent(),"Ce sommet fait partie du sous-arbre  copier.\nChoisissez un autre sommet");	
				}
				else
					JOptionPane.showMessageDialog(this.getParent(),"Ce sommet a dj des fils.\nChoisissez un autre sommet");	

				return;		
			}			
			
			// Cration de fils
			if(sonBoolCreateFils)
			{
				// Copie de l'arbre pour annulation 
				//saCopieArbre = sonArbre.copyArbre();				
				
				dessinerFils(sonSommetTrouve);
				sonSommetTrouve=null;
			}
				
		}
	}
	
	public void mouseMoved(MouseEvent e){}
	public void mouseExited(MouseEvent e) {}				
	public void mouseEntered(MouseEvent e){}
	
	/**
	 * Dessine les 2 fils du sommet pass en paramtre
	 * @param telSommet le sommet auquel on ajoute ses 2 fils
	 */
	private void dessinerFils(UnSommet telSommet)
	{
		UnArbreGraphique telArbre=sonArbre.trouveSommetArbre(telSommet);
		if(telArbre != null)
		{
			// Copie de l'arbre pour annulation 
			saCopieArbre = sonArbre.copyArbre();
			saCopieArbre.getSonSommet().setPosition(sonArbre.getSonSommet().getX(),sonArbre.getSonSommet().getY());
			saCopieArbre.maj_x_y();	

			telArbre.addFils();
			sonArbre.maj_x_y();
			repaint();
		}
	}
	
	/**
	 * Trouve si un sommet est prsent  un point donn
	 * @param telX l'abscisse du point
	 * @param telY l'ordonne du point
	 * @return true si trouv, false sinon
	 */
	private UnSommet detecteSommet(double telX, double telY)
	{
		UnSommet leSommetTrouve = null; 
		sonSommetTrouve = null;	
	
		telX/=ConstantesCreation.SCALE;
		telY/=ConstantesCreation.SCALE;
	
		// On recherche d'abord dans l'arbre existant 	
	 	leSommetTrouve = sonArbre.trouveSommet(telX, telY);
	 	if(leSommetTrouve != null)
	 	{
			sonXTemp=telX;
			sonYTemp=telY;
			
			return leSommetTrouve;
		}
		return null;	 	
	}

	/** 
	 * Dessine le cercle aux coordonnes passes en param
	 * @return true si l'venement doit tre annul, false sinon
	 */
	private boolean bougeSommet(double telX, double telY, UnSommet telSommetTrouve)
	{
		if(telSommetTrouve != sonArbre.getSonSommet())
			return false;
		
		telX/=ConstantesCreation.SCALE;
		telY/=ConstantesCreation.SCALE;

		// Ne pas sortir de la fenetre (annulation de l'vnement)
		// A AMELIORER
		if(telSommetTrouve.getX()+telX-sonXTemp<0 || telSommetTrouve.getX()+telX-sonXTemp+telSommetTrouve.getWidth()> this.getSize().getWidth() ||
	       telSommetTrouve.getY()+telY-sonYTemp<0 || telSommetTrouve.getY()+telY-sonYTemp+telSommetTrouve.getHeight()> this.getSize().getHeight())
		{
			return false;
		}
			
		// Deplacement du cercle
		telSommetTrouve.setPosition(telSommetTrouve.getX()+telX-sonXTemp,telSommetTrouve.getY()+telY-sonYTemp);
		
		sonArbre.maj_x_y();
		
		// Enregistrement des coordonnes de la souris
		sonXTemp=telX;
		sonYTemp=telY;
			
		// Rafraichissement
		repaint();
			
		return true;						 	
	 }

	/**
	 * Annule la cration d'un lment
	 */	
	public void noCreation()
	{
		sonBoolCreateFils = false;
	}	
		
	/**
	 * Lance la cration d'un fils
	 */
	public void createFils()
	{
		sonBoolCreateFils = true;
		sonBoolMove=false;	
	}
	
	/**
	 * Donne l'etat de l'annulation
	 * @return vrai si l'annulation est possible
	 */
	public boolean getUndoState()
	{
		return saCopieArbre != null;	
	}
	
	/**
	 * Annule la dernire action
	 */
	public void doUndo()
	{
		if(saCopieArbre !=null)
		{
			sonArbre = saCopieArbre;
			saCopieArbre = null;
			repaint();
		}
	}
	
	public UnArbreGraphique getCopieArbre()
	{
		return saCopieArbre;
	}
	
	/**
	 * Change l'arbre courant
	 * @param telArbre le nouvelArbre
	 */
	public void setSonArbre(UnArbreGraphique telArbre)
	{
		sonArbre=telArbre;
	}
	
	
}
