package parametres;

import java.awt.Color;
import java.io.*;
import java.util.*;

/**
 * Cette classe permet de grer les paramtrages du programme.
 * Elle stocke les choix de paramtrages entrs par l'utilisateur,
 * et fourni aux autres classes du programmes toutes les informations ncessiares concernant les paramtres.
 * <p>
 * Cette classe ne contient que des champs static et toutes les classes concernes par les paramtrages doivent y
 * avoir accs. Ces classes doivent, <b> chaque fois qu'elles ont besoin d'une couleur ou d'un option</b> 
 * interroger la classe <b>Parametrages</b> pour avoir les couleurs appropries ou l'tat d'activation des options.
 * <p>
 * La classe <b>Parametrages</b> fournit aussi des mthodes permettant de savoir s'il y a eut modification de certains 
 * paramtres concernant tel ou tel onglet, afin de n'y accder rellement que si il y a eut modification.
 *
 * @author Claude JOSEPH-ANGELIQUE
 */ 
public abstract class Parametrages {
	
	/**
	 * Permet d'activer ou de dsactiver l'anti-aliasing
	 * dans l'onglet <b>Cration</b>
	 */
	private static boolean sonAntiAliasing;
	
	/**
	 * Permet de dfinir la largeur de la reprsentation graphique
	 * d'un sommet dans les onglets <b>Cration</b> et <b>Excution</b>
	 */
	private static double saLargeurSommets;
	

	/**
	 * Permet de dfinir la couleur des sommets de l'ABR dans
	 * les onglets <b>Cration</b> et <b>Excution</b>
	 */
	private static Color saCouleurSommets;
	
	/**
	 * Permet de dfinir la couleur de la racine de l'ABR dans
	 * les onglets <b>Cration</b> et <b>Excution</b>
	 */
	private static Color saCouleurRacine;
	
	/**
	 * Permet de dfinir la couleur des feuilles de l'ABR dans
	 * les onglets <b>Cration</b> et <b>Excution</b>
	 */
	private static Color saCouleurFeuilles;	
	
	/**
	 * Permet de dfinir la couleur du texte informatif des sommets
	 * de l'ABR dans les onglets <b>Cration</b> et <b>Excution</b>
	 */
	private static Color saCouleurTexte;
	
	/**
	 * Permet de dfinir la couleur des sommets slectionns dans
	 * les onglets <b>Cration</b> et <b>Excution</b>
	 */
	private static Color saCouleurSelection;
	
	/**
	 * Permet de dfinir la couleur d'un sommet qui sera suivi lors de l'excution
	 * d'un algorithme dans l'onglet <b>Excution</b>
	 */
	private static Color saCouleurSuivi;
	
	/**
	 * Permet de dfinir la couleur de la courbe affiches
	 * dans l'onglet <b>Excution</b>
	 */
	private static Color saCouleurCourbe;
	
	/**
	 * Permet de dfinir la vitesse d'excution des threads excutant
	 * les algorithmes dans l'onglet <b>Excution</b>
	 */
	private static int saVitesseThrd;
	
	/**
	 * Pour savoir si les paramtres de l'onglet excution ont changs
	 */
	private static boolean sonExecutionChange;
	
	/**
	 * Pour savoir si les paramtres de l'onglet cration ont changs
	 */
	private static boolean sonCreationChange;	
	
	/**
	 * Mthodes permettant de d'initialiser les champs d'un objet <b>Parametrages</b>
	 * avec les valeurs par dfaut.
	 */
	public static void init() {
		sonAntiAliasing = false;
		saLargeurSommets = 30;
		saCouleurSommets = Color.blue;
		saCouleurFeuilles = Color.green;
		saCouleurRacine = Color.red;
		saCouleurTexte = Color.black;
		saCouleurSelection = Color.green;
		saCouleurSuivi = Color.yellow;
		saCouleurCourbe = Color.red;
		saVitesseThrd = 2000;
		sonExecutionChange = true;
		sonCreationChange = true;
	}
	
	/**
	 * Permet d'activer ou de dsactiver l'anti-aliasing
	 * dans l'onglet <b>Cration</b>
	 *
	 * @param bool Boolen spcifiant si l'option est active (true) ou non (false) 
	 */
	public static void setAntiAliasing(boolean telBool) {
		
		sonAntiAliasing = telBool;
		sonCreationChange = true;
	}	
	
	/**
	 * Permet de dfinir la couleur des feuilles de l'ABR dans
	 * les onglets <b>Cration</b> et <b>Excution</b>
	 *
	 * @param couleur Couleur spcifie pour une arte  
	 */
	public static void setCoulFeuilles(Color telleCouleur) {
		
		saCouleurFeuilles = telleCouleur;
		sonExecutionChange = true;
		sonCreationChange = true;
	}	

	/**
	 * Permet de dfinir la couleur de la racine de l'ABR dans
	 * les onglets <b>Cration</b> et <b>Excution</b>
	 *
	 * @param couleur Couleur spcifie pour une arte  
	 */
	public static void setCoulRacine(Color telleCouleur) {
		
		saCouleurRacine = telleCouleur;
		sonExecutionChange = true;
		sonCreationChange = true;
	}
	
	/**
	 * Permet de dfinir la couleur des sommets slectionns dans
	 * les onglets <b>Cration</b> et <b>Excution</b>
	 *
	 * @param couleur Couleur spcifie pour une arte  
	 */
	public static void setCoulSelection(Color telleCouleur) {
		
		saCouleurSelection = telleCouleur;
		sonExecutionChange = true;
		sonCreationChange = true;
	}
	
	/**
	 * Permet de dfinir la couleur d'un sommet suivi pendant
	 * l'excution d'un algorithme dans l'onglet <b>Excution</b>
	 *
	 * @param couleur Couleur spcifie pour une arte  
	 */
	public static void setCoulSuivi(Color telleCouleur) {
		
		saCouleurSuivi = telleCouleur;
		sonExecutionChange = true;
	}
	
	/**
	 * Permet de dfinir la couleur de la courbe
	 * dans l'onglet <b>Excution</b>
	 *
	 * @param couleur Couleur spcifie pour une arte  
	 */
	public static void setCoulCourbe(Color telleCouleur) {
		
		saCouleurCourbe = telleCouleur;
		sonExecutionChange = true;
	}
	
	/**
	 * Permet de dfinir la couleur des sommets de l'ABR dans
	 * les onglets <b>Cration</b> et <b>Excution</b>
	 *
	 * @param couleur Couleur spcifie pour une arte  
	 */
	public static void setCoulSommets(Color telleCouleur) {
		
		saCouleurSommets = telleCouleur;
		sonExecutionChange = true;
		sonCreationChange = true;
	}
	
	/**
	 * Permet de dfinir la couleur du texte informatif des sommets
	 * de l'ABR dans les onglets <b>Cration</b> et <b>Excution</b>
	 *
	 * @param couleur Couleur spcifie pour une arte  
	 */
	public static void setCoulTexte(Color telleCouleur) {
		
		saCouleurTexte = telleCouleur;
		sonExecutionChange = true;
		sonCreationChange = true;
	}
	
	/**
	 * Permet de dfinir la largeur de la reprsentation graphique
	 * d'un sommet dans les onglets <b>Cration</b> et <b>Excution</b>
	 *
	 * @param largeur Largeur spcifie pour un sommet
	 */
	public static void setDiametreSommets(double telleLargeur) {
		
		saLargeurSommets = telleLargeur;
		sonExecutionChange = true;
		sonCreationChange = true;
	}
	
	/**
	 * Permet de dfinir la vitesse d'un thread
	 * dans l'onglet <b>Excution</b>
	 *
	 * @param vitesse Coefficient spcifiant la vitesse d'un thread  
	 */
	public static void setVitesse(int telleVitesse) {
		
		saVitesseThrd = telleVitesse;
		sonExecutionChange = true;
	}
		
	/**
	 * Pour obtenir l'tat de l'option d'anti-aliasing.
	 *
	 * @return Un boolen indiquant si l'option est active ou pas.
	 */
	public static boolean getAntiAliasing() {
		
		return sonAntiAliasing;
	}
		
	/**
	 * Pour obtenir la couleur des feuilles.
	 *
	 * @return Couleur des feuilles.
	 */
	public static Color getCoulFeuilles() {
		
		return saCouleurFeuilles;
	}
	
	/**
	 * Pour obtenir la couleur de la racine.
	 *
	 * @return Couleur de la racine.
	 */
	public static Color getCoulRacine() {
		
		return saCouleurRacine;
	}
	
	/**
	 * Pour obtenir la couleur des sommets.
	 *
	 * @return Couleur des sommets.
	 */
	public static Color getCoulSommets() {
		
		return saCouleurSommets;
	}	
	
	/**
	 * Pour obtenir la couleur des sommets slectionns dans les panels o sont dessins les arbres.
	 *
	 * @return La couleur des sommets slectionns.
	 */
	public static Color getCoulSelection() {
		
		return saCouleurSelection;
	}
	
	/**
	 * Pour obtenir la couleur du sommet suivi.
	 *
	 * @return La couleur du sommet suivi.
	 */
	public static Color getCoulSuivi() {
		
		return saCouleurSuivi;
	}
	
	/**
	 * Pour obtenir la couleur de la courbe.
	 *
	 * @return La couleur de la courbe.
	 */
	public static Color getCoulCourbe() {
		
		return saCouleurCourbe;
	}
	
	/**
	 * Pour obtenir la couleur du texte informatif des sommets.
	 *
	 * @return La couleur du texte.
	 */
	public static Color getCoulTexte() {
		
		return saCouleurTexte;
	}
	
	
	/**
	 * Mthode permettant de savoir si les paramtres concernants l'onglet cration 
	 * ont t modifis depuis le prcdent appel  cette mthode.
	 *
	 * @return true si il y a eut modification, false sinon.
	 */
	public static boolean creationChange() {
		
		boolean retour = sonCreationChange;
		
		sonCreationChange = false;
		
		return retour;
	}
	
	/**
	 * Mthode permettant de savoir si les paramtres concernants l'onglet excution 
	 * ont t modifis depuis le prcdent appel  cette mthode.
	 *
	 * @return true si il y a eut modification, false sinon.
	 */
	public static boolean executionChange() {
		
		boolean retour = sonExecutionChange;
		
		sonExecutionChange = false;
				
		return retour;
	}
	
	/**
	 * Pour obtenir la vitesse d'excution des threads.
	 *
	 * @return Vitesse des threads.
	 */
	public static int getVitesse() {
		
		return saVitesseThrd;
	}
	
	/**
	 * Pour obtenir la largeur d'un sommet.
	 *
	 * @return Largeur d'un sommet.
	 */
	public static double getDiametreSommet() {
		
		return saLargeurSommets;
	}
	
	/**
	 * Pour obtenir les paramtres de l'onglet cration
	 *
	 * @return Un objet <b>ParametresCreation</b> regroupant les paramtres
	 */
	public static ParametresCreation getParamCreation() {
		
		ParametresCreation leParam = new ParametresCreation(saCouleurRacine, saCouleurSommets, saCouleurSelection,
		saCouleurFeuilles, saCouleurTexte, saLargeurSommets, sonAntiAliasing);
		
		return leParam;
	}
	
	/**
	 * Pour obtenir les paramtres de l'onglet excution
	 *
	 * @return Un objet <b>ParametresExecution</b> regroupant les paramtres
	 */
	public static ParametresExecution getParamExecution() {
		
		ParametresExecution leParam = new ParametresExecution(saCouleurRacine, saCouleurSommets, saCouleurSelection,
		saCouleurFeuilles, saCouleurTexte, saCouleurSuivi, saCouleurCourbe, saLargeurSommets, saVitesseThrd);
		
		return leParam;
	}
	
	/**
	 * Pour sauvegarder les paramtres actuels sous forme de fichier texte
	 * 
	 * @param telNomFichier Nom du fichier de sauvegarde
	 */
	public static void Sauver(String telNomFichier)	throws IOException {
		
		PrintWriter laSortie = new PrintWriter(new FileWriter(telNomFichier));
		
		laSortie.println("courbe = "+saCouleurCourbe.getRed()+" "+saCouleurCourbe.getGreen()+" "+saCouleurCourbe.getBlue());
		laSortie.println("feuille = "+saCouleurFeuilles.getRed()+" "+saCouleurFeuilles.getGreen()+" "+saCouleurFeuilles.getBlue());
		laSortie.println("racine = "+saCouleurRacine.getRed()+" "+saCouleurRacine.getGreen()+" "+saCouleurRacine.getBlue());
		laSortie.println("selection = "+saCouleurSelection.getRed()+" "+saCouleurSelection.getGreen()+" "+saCouleurSelection.getBlue());
		laSortie.println("sommets = "+saCouleurSommets.getRed()+" "+saCouleurSommets.getGreen()+" "+saCouleurSommets.getBlue());
		laSortie.println("suivi = "+saCouleurSuivi.getRed()+" "+saCouleurSuivi.getGreen()+" "+saCouleurSuivi.getBlue());
		laSortie.println("texte = "+saCouleurTexte.getRed()+" "+saCouleurTexte.getGreen()+" "+saCouleurTexte.getBlue());
		laSortie.println("diametre_sommet = "+saLargeurSommets);
		laSortie.println("vitesse_thread = "+saVitesseThrd);
		laSortie.println("anti-aliasing = "+sonAntiAliasing);
				
		laSortie.close();
	}
	
	/**
	 * Pour charger des paramtres depuis un fichier texte
	 * 
	 * @param telNomFichier Nom du fichier  charger
	 */
	public static void Charger(String telNomFichier) throws IOException {
		
		String laLigne;
		String laChaine;
		int leRouge, leVert, leBleu, laVit;
		boolean leBool;
		double leDiametre;
		StringTokenizer leTok;
		BufferedReader lEntree = new BufferedReader(new FileReader(telNomFichier));
		
		laLigne = lEntree.readLine();
		leTok = new StringTokenizer(laLigne, " ");
		leTok.nextToken();
		leTok.nextToken();
		leRouge = Integer.parseInt(leTok.nextToken());
		leVert = Integer.parseInt(leTok.nextToken());
		leBleu = Integer.parseInt(leTok.nextToken());
		saCouleurCourbe = new Color(leRouge, leVert, leBleu);
		
		System.out.println(laLigne);
		
		laLigne = lEntree.readLine();
		leTok = new StringTokenizer(laLigne, " ");
		leTok.nextToken();
		leTok.nextToken();
		leRouge = Integer.parseInt(leTok.nextToken());
		leVert = Integer.parseInt(leTok.nextToken());
		leBleu = Integer.parseInt(leTok.nextToken());
		saCouleurFeuilles = new Color(leRouge, leVert, leBleu);
		
		System.out.println(laLigne);
		
		laLigne = lEntree.readLine();
		leTok = new StringTokenizer(laLigne, " ");
		leTok.nextToken();
		leTok.nextToken();
		leRouge = Integer.parseInt(leTok.nextToken());
		leVert = Integer.parseInt(leTok.nextToken());
		leBleu = Integer.parseInt(leTok.nextToken());
		saCouleurRacine = new Color(leRouge, leVert, leBleu);
		
		System.out.println(laLigne);
		
		laLigne = lEntree.readLine();
		leTok = new StringTokenizer(laLigne, " ");
		leTok.nextToken();
		leTok.nextToken();
		leRouge = Integer.parseInt(leTok.nextToken());
		leVert = Integer.parseInt(leTok.nextToken());
		leBleu = Integer.parseInt(leTok.nextToken());
		saCouleurSelection = new Color(leRouge, leVert, leBleu);
		
		System.out.println(laLigne);
		
		laLigne = lEntree.readLine();
		leTok = new StringTokenizer(laLigne, " ");
		leTok.nextToken();
		leTok.nextToken();
		leRouge = Integer.parseInt(leTok.nextToken());
		leVert = Integer.parseInt(leTok.nextToken());
		leBleu = Integer.parseInt(leTok.nextToken());
		saCouleurSommets = new Color(leRouge, leVert, leBleu);
		
		System.out.println(laLigne);
		
		laLigne = lEntree.readLine();
		leTok = new StringTokenizer(laLigne, " ");
		leTok.nextToken();
		leTok.nextToken();
		leRouge = Integer.parseInt(leTok.nextToken());
		leVert = Integer.parseInt(leTok.nextToken());
		leBleu = Integer.parseInt(leTok.nextToken());
		saCouleurSuivi = new Color(leRouge, leVert, leBleu);
		
		System.out.println(laLigne);
		
		laLigne = lEntree.readLine();
		leTok = new StringTokenizer(laLigne, " ");
		leTok.nextToken();
		leTok.nextToken();
		leRouge = Integer.parseInt(leTok.nextToken());
		leVert = Integer.parseInt(leTok.nextToken());
		leBleu = Integer.parseInt(leTok.nextToken());
		saCouleurTexte = new Color(leRouge, leVert, leBleu);
		
		System.out.println(laLigne);
		
		laLigne = lEntree.readLine();
		leTok = new StringTokenizer(laLigne, " ");
		leTok.nextToken();
		leTok.nextToken();
		leDiametre = Double.parseDouble(leTok.nextToken());
		saLargeurSommets = leDiametre;
		
		System.out.println(laLigne);
		
		laLigne = lEntree.readLine();
		leTok = new StringTokenizer(laLigne, " ");
		leTok.nextToken();
		leTok.nextToken();
		laVit = Integer.parseInt(leTok.nextToken());
		saVitesseThrd = laVit;
		
		System.out.println(laLigne);
		
		laLigne = lEntree.readLine();
		leTok = new StringTokenizer(laLigne, " ");
		leTok.nextToken();
		leTok.nextToken();
		leBool = (Boolean.valueOf(leTok.nextToken())).booleanValue();
		sonAntiAliasing = leBool;
		
		System.out.println(laLigne);		
	}
	
}
