package structuresgraphique;

import java.awt.Graphics2D;
import java.awt.Color;
import java.util.Vector;

/**
 * Cette classe rprsente la structure d'un arbre destine
 *  tre afficher graphiquement
 * @author Matthieu LOMON - Nicolas Maupu
 */
public class UnArbreGraphique
{
	private UnSommet sonSommet = null;
	
	// Les 2 arbres fils
	private UnArbreGraphique sonFilsGauche  = null;
	private UnArbreGraphique sonFilsDroit   = null;
	
	// Les donnes des 2 aretes vers les fils
	private ListeStockage sesDonneesAreteGauche = null;
	private ListeStockage sesDonneesAreteDroite = null;
	
	// Couleur aretes
	private Color saCouleurAreteGauche = ConstantesFormes.ARETE_COLOR;
	private Color saCouleurAreteDroite = ConstantesFormes.ARETE_COLOR;
	
	// Etage du sommet courant (racine => etage == 1)
	private int sonEtage;
	
	// Le numro du sommet
	private int sonNumSommet;

	/**
	 * Pour construire la racine au point par dfaut
	 */
	public UnArbreGraphique()
	{	
		this(ConstantesFormes.X_DEPART,ConstantesFormes.Y_DEPART);
	}
	
	/**
	 * Pour construire la racine au point pass en paramtre
	 */
	private UnArbreGraphique(double telX, double telY)
	{	
		this(telX,telY,0,1);
	}
		
	/**
	 * Pour construire le reste de l'arbre
	 */
	private UnArbreGraphique(double x, double y, int telEtage, int telNumSommet)
	{
		sonNumSommet=telNumSommet;
		sonSommet = UnSommet.retourneSommet(x,y,sonNumSommet,ConstantesFormes.SOMMET_COLOR_AUTRES);
		sonEtage = telEtage + 1;
		maj_x_y();
	}

	/**
	 * Retourne le fils gauche
	 * @return le fils gauche
	 */		
	public UnArbreGraphique getSonFilsGauche()
	{
		return sonFilsGauche;
	}
	
	/**
	 * Retourne le fils droit
	 * @return le fils droit
	 */	
	public UnArbreGraphique getSonFilsDroit()
	{
		return sonFilsDroit;
	}
	
	/**
	 * Ajoute un fils gauche et un fils droit
	 */
	public void addFils()
	{
		if(estFeuille())
		{
			sonFilsGauche = new UnArbreGraphique(ConstantesFormes.X_DEPART,sonSommet.getY()+ConstantesFormes.Y_DEPART, sonEtage, sonNumSommet*2);
			sonFilsDroit  = new UnArbreGraphique(ConstantesFormes.X_DEPART,sonSommet.getY()+ConstantesFormes.Y_DEPART, sonEtage, sonNumSommet*2+1);
			
			sesDonneesAreteGauche = new ListeStockage();
			sesDonneesAreteDroite = new ListeStockage();
		}
	}
	
	/**
	 * Ajoute des arbres en tant que fils (utile pour dplacer un sous-arbre)
	 * @param telFilsGauche l'arbre gauche  attacher
	 * @param telFilsDroit l'arbre droit  attacher
	 * @param telAncienArbre l'arbre sur lequel on prend les donnes
	 */
	public void addFils(UnArbreGraphique telFilsGauche, UnArbreGraphique telFilsDroit, UnArbreGraphique telAncienArbre)
	{
		if(estFeuille())
		{
			sonFilsGauche = telFilsGauche;
			sonFilsDroit  = telFilsDroit;
			
			sesDonneesAreteGauche=telAncienArbre.getSesDonneesAreteGauche();
			sesDonneesAreteDroite=telAncienArbre.getSesDonneesAreteDroite();
		}
	}

	/**
	 * Ajoute des arbres en tant que fils (utile pour dplacer un sous-arbre)
	 * @param telFilsGauche l'arbre gauche  attacher
	 * @param telFilsDroit l'arbre droit  attacher
	 */	
	public void addFils(UnArbreGraphique telFilsGauche, UnArbreGraphique telFilsDroit)
	{
		if(estFeuille())
		{
			sonFilsGauche = telFilsGauche;
			sonFilsDroit  = telFilsDroit;
			
			sesDonneesAreteGauche=new ListeStockage();
			sesDonneesAreteDroite=new ListeStockage();
		}
	}
		
	/**
	 * Supprime les 2 fils
	 */
	public void supprimeFils()
	{
		// Les 2 arbres fils
		sonFilsGauche = null;
		sonFilsDroit = null;
		
		// Les donnes des 2 aretes
		sesDonneesAreteGauche=null;
		sesDonneesAreteGauche=null;
		
		// Les couleurs des 2 aretes
		saCouleurAreteGauche = null;
		saCouleurAreteDroite = null;
	}
	
	/**
	 * Retourne le sommet
	 * @return le sommet
	 */
	public UnSommet getSonSommet()
	{
		return sonSommet;	
	}
	
	/**
	 * Trouve et retourne le sommet se trouvant au point donn
	 * @param telX l'abscisse du point
	 * @param telY l'ordonne du point
	 * @return le sommet trouv
	 */
	public UnSommet trouveSommet(double telX, double telY)
	{
		UnSommet leSommetTrouve = null;
		
		if(sonSommet.contains(telX,telY))
			return sonSommet;
			
		if(!estFeuille())
		{
			leSommetTrouve = sonFilsGauche.trouveSommet(telX, telY);
			if(leSommetTrouve != null)
				return leSommetTrouve;

			leSommetTrouve = sonFilsDroit.trouveSommet(telX, telY);
			if(leSommetTrouve != null)
				return leSommetTrouve;
		}
		return null;		
	}
	
	/**
	 * Trouve et retourne le sommet ayant un id donn
	 * @param telX l'abscisse du point
	 * @param telY l'ordonne du point
	 * @return le sommet trouv
	 */
	public UnSommet trouveSommet(int telNb)
	{
		UnSommet leSommetTrouve = null;
		
		if(sonSommet.getSonId() == telNb)
			return sonSommet;
			
		if(!estFeuille())
		{
			leSommetTrouve = sonFilsGauche.trouveSommet(telNb);
			if(leSommetTrouve != null)
				return leSommetTrouve;

			leSommetTrouve = sonFilsDroit.trouveSommet(telNb);
			if(leSommetTrouve != null)
				return leSommetTrouve;
		}
		return null;		
	}	
	
	/**
	 * Trouve le sommet pass en paramtre et retourne l'arbre
	 * duquel il est racine
	 * @param telSommet le sommet  trouver
	 * @return l'arbre correspondant
	 */
	public UnArbreGraphique trouveSommetArbre(UnSommet telSommet)
	{
		if(telSommet != null)
		{			
			if(sonSommet == telSommet)
				return this;
	
			if(!estFeuille())
			{
				UnArbreGraphique lArbreTrouve = null;
				
				lArbreTrouve = sonFilsGauche.trouveSommetArbre(telSommet);
				if(lArbreTrouve != null)
					return lArbreTrouve;
					
				lArbreTrouve = sonFilsDroit.trouveSommetArbre(telSommet);
				if(lArbreTrouve != null)
					return lArbreTrouve;
			}
		}		
		return null;		
	}
	
	/**
	 * Supprime le sommet et tous ses fils
	 * @param telSommet le sommet  supprimer
	 * @return true si trouv, false sinon
	 */
	public void supprimerSommet(UnSommet telSommet)
	{
		if(sonSommet == telSommet)
		{
			sonSommet = null;
			supprimeFils();
		}
		else
		{
			if(!estFeuille())
			{
				if(sonFilsGauche.getSonSommet() == telSommet || sonFilsDroit.getSonSommet() == telSommet)
				{
					supprimeFils();
				}
				else
				{
					sonFilsGauche.supprimerSommet(telSommet);
					sonFilsDroit.supprimerSommet(telSommet);
				}
			}
		}
	}

	/**
	 * Dessine l'arbre
	 * @param g2d le contexte d'affichage
	 */	
	public void dessineArbre(Graphics2D g2d)
	{
		// Les aretes
		g2d.setColor(ConstantesFormes.ARETE_COLOR);
		genererAretes(g2d);	
				
		// Dessine le sommet
		sonSommet.dessineSommet(g2d);
		
		// Les 2 fils
		if(!estFeuille())
		{
			sonFilsGauche.dessineArbre(g2d);
			sonFilsDroit.dessineArbre(g2d);
		}		
	}
	
	/**
	 * Dessine les artes entre les sommets
	 * @param g2d le contexte d'affichage
	 */
	private void genererAretes(Graphics2D g2d)
	{
		int x1,x2,y1,y2;
			
		if(!estFeuille())
		{	
			g2d.setColor(saCouleurAreteGauche);
			x1=(int)sonSommet.getCenterX();
			y1=(int)sonSommet.getCenterY();
			x2=(int)sonFilsGauche.getSonSommet().getCenterX();
			y2=(int)sonFilsGauche.getSonSommet().getCenterY();
			g2d.drawLine(x1,y1,x2,y2);

			g2d.setColor(saCouleurAreteDroite);
			x1=(int)sonSommet.getCenterX();
			y1=(int)sonSommet.getCenterY();
			x2=(int)sonFilsDroit.getSonSommet().getCenterX();
			y2=(int)sonFilsDroit.getSonSommet().getCenterY();
			g2d.drawLine(x1,y1,x2,y2);
		}
	}

	/**
	 * Retourne la hauteur maximale de l'arbre
	 * @return la hauteur de l'arbre
	 */
	public int getSaHauteur()
	{
		if(!estFeuille())
			return 1+Math.max(sonFilsGauche.getSaHauteur(),sonFilsDroit.getSaHauteur());
		else
			return 1;
	}
	
	/**
	 * Met  jour les positions des sommet de l'arbre
	 */
	public void maj_x_y()
	{
		double xdepart, ydepart, xg, xd;
			
		xdepart=sonSommet.getX();	
		ydepart=sonSommet.getY();
		
		// Coloration selon le type	
		if(estRacine())
			sonSommet.setSaCouleur(ConstantesFormes.SOMMET_COLOR_RACINE);
		else
			if(estFeuille())
				sonSommet.setSaCouleur(ConstantesFormes.SOMMET_COLOR_FEUILLE);
			else
				sonSommet.setSaCouleur(ConstantesFormes.SOMMET_COLOR_AUTRES);
			
		xg = xdepart-Math.pow(2,getSaHauteur()) * ConstantesFormes.ESPACE;
		xd = xdepart+Math.pow(2,getSaHauteur()) * ConstantesFormes.ESPACE;
			
		if(!estFeuille())
		{
			sonFilsGauche.maj_x_y_fils(xg, ydepart, 2*sonNumSommet);
			sonFilsDroit.maj_x_y_fils(xd, ydepart, 2*sonNumSommet+1);
		}
	}
	
	/**
	 * Met  jour les positions de tout les fils
	 * @param telX l'abscisse du sommet courant
	 * @param telY l'ordonne du sommet courant
	 */
	private void maj_x_y_fils(double telX, double telY, int telNum)
	{
		double xg, xd;
		
		sonSommet.setPosition(telX,telY+ConstantesFormes.Y_DEPART);
			
		xg = sonSommet.getX()-Math.pow(2,getSaHauteur())*ConstantesFormes.ESPACE;
		xd = sonSommet.getX()+Math.pow(2,getSaHauteur())*ConstantesFormes.ESPACE;
		sonNumSommet=telNum;
		sonSommet.setSonId(sonNumSommet);

		if(!estFeuille())
		{
			sonSommet.setSaCouleur(ConstantesFormes.SOMMET_COLOR_AUTRES);
			
			sonFilsGauche.maj_x_y_fils(xg,telY+ConstantesFormes.Y_DEPART, 2*sonNumSommet);
			sonFilsDroit.maj_x_y_fils(xd,telY+ConstantesFormes.Y_DEPART, 2*sonNumSommet+1);
		}
		else
			sonSommet.setSaCouleur(ConstantesFormes.SOMMET_COLOR_FEUILLE);			
	}

	/**
	 * Test si l'arbre est vide
	 * @return true si l'arbre est vide, faux sinon
	 */
	public boolean estVide()
	{
		return sonSommet == null || (sonFilsGauche==null && sonFilsDroit == null);	
	}
	
	/**
	 * Retourne une copie de l'arbre
	 * @return la copie de l'arbre
	 */
	public UnArbreGraphique copyArbre()
	{
		UnArbreGraphique leNouvelArbre = new UnArbreGraphique();
		
		leNouvelArbre.sonSommet = sonSommet.copySommet();
		//leNouvelArbre.sonSommet.setPosition(sonSommet.getX(), sonSommet.getY());
		
		if(sonEtage == 1)
			leNouvelArbre.sonSommet.setSonId(1);
			
		if(!estFeuille())
		{	
			leNouvelArbre.addFils();
			
			leNouvelArbre.sesDonneesAreteGauche=(ListeStockage)sesDonneesAreteGauche.clone();
			leNouvelArbre.sesDonneesAreteDroite=(ListeStockage)sesDonneesAreteDroite.clone();
			
			leNouvelArbre.sonFilsGauche= sonFilsGauche.copyArbre();
			leNouvelArbre.sonFilsDroit = sonFilsDroit.copyArbre();			
		}

		leNouvelArbre.getSonSommet().setPosition(ConstantesFormes.X_DEPART,ConstantesFormes.Y_DEPART);
		leNouvelArbre.maj_x_y();
		
		return leNouvelArbre;
	}

	/**
	 * Met  jour la couleur du texte de chaque sommet de l'arbre
	 * @param telleCouleur la couleur du texte des sommets
	 */	
	public void majColorTexte(Color telleCouleur)
	{
		sonSommet.setSaCouleurTexte(telleCouleur);
			
		if(!estFeuille())
		{				
			sonFilsGauche.majColorTexte(telleCouleur);
			sonFilsDroit.majColorTexte(telleCouleur);
		}	
	}

	/**
	 * Met  jour la couleur des aretes
	 * @param telleCouleur la couleur des aretes
	 */	
	public void majColorArete(Color telleCouleur)
	{
		if(!estFeuille())
		{	
			saCouleurAreteGauche = telleCouleur;
			saCouleurAreteDroite = telleCouleur;
		
			sonFilsGauche.majColorArete(telleCouleur);
			sonFilsDroit.majColorArete(telleCouleur);
		}	
	}


	/**
	 * Met  jour le diamtre de chaque sommet de l'arbre
	 * @param telDiametre le nouveau diamtre des sommets
	 */		
	public void majDiametreSommets(double telDiametre)
	{
		ConstantesFormes.LARGEUR_ELL = telDiametre;
		ConstantesFormes.ESPACE      = ConstantesFormes.LARGEUR_ELL / ConstantesFormes.RAPPORT_ESPACE_LARGEUR;
		ConstantesFormes.Y_DEPART    = ConstantesFormes.LARGEUR_ELL * ConstantesFormes.COEFF_HAUTEUR_ESPACEMENT;
		
		sonSommet.setDiametre(telDiametre);
			
		if(!estFeuille())
		{				
			sonFilsGauche.majDiametreSommets(telDiametre);
			sonFilsDroit.majDiametreSommets(telDiametre);	
		}
		
	}
	
	/**
	 * Verifie si le sommet de l'arbre courant est une racine ou non
	 * @return true si c'est une racine, false sinon
	 */
	public boolean estRacine()
	{
		return (sonEtage == 1);	
	}
	
	/**
	 * Verifie si le sommet de l'arbre courant est une feuille ou non
	 * @return true si c'est une feuille, false sinon
	 */
	public boolean estFeuille()
	{
		return (sonFilsGauche == null && sonFilsDroit == null) ? true : false;
	}

	/**
	 * Retourne la liste de stockage de l'arete gauche
	 * @return la liste de stockage de l'arete gauche
	 */
	public ListeStockage getSesDonneesAreteGauche()
	{
		return sesDonneesAreteGauche;	
	}
	
	/**
	 * Retourne la liste de stockage de l'arete droite
	 * @return la liste de stockage de l'arete droite
	 */
	public ListeStockage getSesDonneesAreteDroite()
	{
		return sesDonneesAreteDroite;	
	}
	
	/**
	 * Change la liste de stockage de l'arete gauche
	 * @param la nouvelle liste
	 */
	public void setSesDonneesAreteGauche(ListeStockage telleListe)
	{
		if(!estFeuille())
			sesDonneesAreteGauche=telleListe;	
	}	
	
	/**
	 * Change la liste de stockage de l'arete gauche
	 * @param la nouvelle liste
	 */
	public void setSesDonneesAreteDroite(ListeStockage telleListe)
	{
		if(!estFeuille())
			sesDonneesAreteDroite=telleListe;	
	}	
	
	/**
	 * Retourne la couleur de l'arete gauche du sommet
	 * @return Color la couleur de l'arte gauche
	 */
	public Color getSaCouleurAreteGauche()
	{
		return (sonFilsGauche != null) ? saCouleurAreteGauche : null;
	}
	 
	/**
	 * Retourne la couleur de l'arete droite du sommet
	 * @return Color la couleur de l'arte droite
	 */
	public Color getSaCouleurAreteDroite()
	{
		return (sonFilsDroit != null) ? saCouleurAreteDroite : null;
	}
	 
	/**
	 * Change la couleur de l'arte gauche
	 * @param telleCouleur la nouvelle couleur de l'arte gauche
	 */
	public void setSaCouleurAreteGauche(Color telleCouleur)
	{
		if(!estFeuille())
			saCouleurAreteGauche = telleCouleur;
	}
	
	/**
	 * Change la couleur de l'arte droite
	 * @param telleCouleur la nouvelle couleur de l'arte droite
	 */
	public void setSaCouleurAreteDroite(Color telleCouleur)
	{
		if(!estFeuille())
			saCouleurAreteDroite = telleCouleur;
	}
}
