package structuresgraphique;

import java.awt.geom.*;
import java.util.Vector;
import java.awt.Color;
import java.awt.Graphics2D;

import java.awt.Font;

/**
 * Cette classe permet de reprsenter un sommet. Elle est
 * est compose d'un centre et d'une largeur(diamtre). Cette
 * classe hrite de la classe Ellipse.Double pour tre facilement
 * dessine dans un panel avec les mthodes de Graphics2D
 * @author Matthieu LOMON - Nicolas Maupu
 */
public class UnSommet extends Ellipse2D.Double
{
	private int sonId;
	private Color saCouleur=null;
	private Color saCouleurTexte=null;
	private ListeStockage sesDonnees = new ListeStockage();
	
   	/**
	 * Cre le sommet
	 * @param telX Abscisse du point haut gauche
	 * @param telY Ordonne du point haut gauche
	 * @param telW Diamtre du sommet
	 * @param telleCouleur la couleur du sommet
     */		
	private UnSommet(double telX, double telY, double telW, int telId, Color telleCouleur)
	{
		super(telX,telY,telW,telW);
		sonId = telId;
		saCouleur = telleCouleur;
		saCouleurTexte = ConstantesFormes.SOMMET_COLOR_TEXTE;
		setPosition(getCenterX(),getCenterY());
		
		// Ajout de la donne par dfaut
		setStrahler(0);	
	}
	
	/**
	 * Recupre la couleur du sommet
	 * @return la couleur du sommet
	 */
	public Color getSaCouleur()
	{
		return saCouleur;
	}
	
	/**
	 * Change la couleur du sommet
	 * @param telleCouleur la nouvelle couleur du sommet
	 */
	public void setSaCouleur(Color telleCouleur)
	{
		saCouleur = telleCouleur;
	}

   	/**
	 * Change la position du sommet
	 * @param telX Abscisse du nouveau point
	 * @param telY Ordonne du nouveau point
     */		
	public void setPosition(double telX, double telY)
	{
		x=telX;
		y=telY;
	}

	/**
     * Retourne l'identifiant du sommet
     * @return le texte
     */
    public int getSonId()
    {
    	return sonId;	
    }

    /**
     * Change le texte  afficher sur le sommet
     * @param telId le nouveau texte
     */    
    public void setSonId(int telId)
    {
    	sonId = telId;
    }

    /**
     * Retourne la couleur du texte
     * @return la couleur du texte
     */
    public Color getSaCouleurTexte()
    {
    	return saCouleurTexte;	
    }
    
    /**
     * Change la couleur du texte
     * @param telleCouleur la nouvelle couleur du texte
     */
    public void setSaCouleurTexte(Color telleCouleur)
    {
    	saCouleurTexte = telleCouleur;
    }
    
   	/**
	 * Retourne un sommet avec une taille prdfinie
	 * @param telX Abscisse du point haut gauche
	 * @param telY Ordonne du point haut droit
	 * @param telTexte le texte  ecrire sur le sommet
	 * @param telleCouleur la couleur du sommet
     */	 
	public static UnSommet retourneSommet(double telX, double telY,  int telId, Color telleCouleur)
	{	
		return new UnSommet(telX, telY, ConstantesFormes.LARGEUR_ELL, telId, telleCouleur);
	}
	
	/**
	 * Dessine le sommet
	 * @param g2d le contexte d'affichage
	 */	
	public void dessineSommet(Graphics2D g2d)
	{	
		// Le sommet
  		g2d.setColor(getSaCouleur());
  		g2d.fill(this);
  		
  		// Le contour noir du sommet
  		g2d.setColor(ConstantesFormes.SOMMET_COLOR_CONTOUR);
		g2d.draw(this);
  			
  		// La chaine identifiant le sommet	
		g2d.setColor(getSaCouleurTexte());
	
		// Gestion de la taille du texte
		Font laFontBak = g2d.getFont();
		g2d.setColor(getSaCouleurTexte());
		
		if(sonId<100)
		{	
			Font laNewFont = new Font(laFontBak.getName(), laFontBak.BOLD, (int)(ConstantesFormes.LARGEUR_ELL*0.8));
			g2d.setFont(laNewFont);
		
			Rectangle2D leRectText = laNewFont.getStringBounds(""+sonId,g2d.getFontRenderContext());
			leRectText.setRect((int)(this.getX()+ConstantesFormes.LARGEUR_ELL/2-leRectText.getWidth()/2),this.getY(), leRectText.getWidth(), leRectText.getHeight());

			g2d.drawString(""+sonId,(int)(leRectText.getX()+0.9),(int)(leRectText.getY()+leRectText.getHeight()-ConstantesFormes.LARGEUR_ELL/6.8));
		}
		else
		{
			Font laNewFont = new Font(laFontBak.getName(), laFontBak.BOLD, (int)(ConstantesFormes.LARGEUR_ELL*0.4));
			g2d.setFont(laNewFont);

			Rectangle2D leRectText = laNewFont.getStringBounds(""+sonId,g2d.getFontRenderContext());
			leRectText.setRect((int)(this.getX()+ConstantesFormes.LARGEUR_ELL/2-leRectText.getWidth()/2),this.getY(), leRectText.getWidth(), leRectText.getHeight());
			
			g2d.drawString(""+sonId,(int)(leRectText.getX()+0.9),(int)(leRectText.getY()+leRectText.getHeight()+ConstantesFormes.LARGEUR_ELL/5.66));
		}
		g2d.setFont(laFontBak);
	}
	
	/**
	 * Retourne une copie du sommet
	 * @return la copie du sommet
	 */
	public UnSommet copySommet()
	{
		UnSommet leNouveauSommet = retourneSommet(getX(), getY(), sonId, saCouleur);
		
		leNouveauSommet.setSaCouleurTexte(saCouleurTexte);
		leNouveauSommet.sesDonnees=(ListeStockage)getSesDonnees().clone();
		
		return leNouveauSommet;
	}
	
	/**
	 * Change le diametre du sommet
	 * @param telDiametre
	 */
	public void setDiametre(double telDiametre)
	{
		super.width = telDiametre;
		super.height= telDiametre;
	}
	
	/**
	 * Retourne la liste contenant les donnes
	 * @return ListeStockage le vecteur des donnes
	 */
	public ListeStockage getSesDonnees()
	{
		return sesDonnees;
	}
	
	/**
	 * Retourne le vecteur contenant les donnes
	 * @return Vector le vecteur des donnes
	 */
	public Vector getVectorSesDonnees()
	{
		if(sesDonnees.size()==1)
			return null;
		
		Vector leVecteur=new Vector();
		
		for(int i=1; i<sesDonnees.size();i++)
		{
			String [] leTabTemp=new String[2];
			Stockage leStockageTemp=sesDonnees.getStockage(i);
			
			leTabTemp[0]=new String(leStockageTemp.getNom());
			leTabTemp[1]=new String(leStockageTemp.getContenu());
			
			leVecteur.add(leTabTemp);		
		}
		return leVecteur;
	}

	/**
	 * Stocke le nombre de Strahler
	 * @param telShraler le nombre de Strahler
	 */
	public void setStrahler(int telStrahler)
	{
		if(sesDonnees.getStockage(0) == null)
			sesDonnees.addStockage(new Stockage("Strahler",telStrahler+""));
		else
		{
			sesDonnees.getStockage(0).setNom("Strahler");
			sesDonnees.getStockage(0).setContenu(telStrahler+"");
		}
	}
	
	/**
	 * Recupre le nombre de Strahler
	 */
	 public int getStrahler()
	 {
	 	return Integer.parseInt(sesDonnees.getStockage(0).getContenu());
	 }
	
	/**
	 * Ajoute les donnes  la liste de stockage
	 * @param tellesDonnees le vecteur contenant les donns  copier
	 */
	public void addData(Vector tellesDonnees)
	{
		if(tellesDonnees !=null)
		{
			for(int i=0;i<tellesDonnees.size();i++)
			{
				String [] laLigne=(String[])tellesDonnees.get(i);
				//if(laLigne[0] != null && laLigne[1]!=null)
					sesDonnees.addStockage(new Stockage(new String(laLigne[0]),new String(laLigne[1])));

			}
		}
	}	
}
