package editeurarbre;

import javax.swing.JTable;
import javax.swing.table.TableModel;
import javax.swing.table.TableColumn;
import structuresgraphique.ListeStockage;
import structuresgraphique.Stockage;


public class Tableau extends JTable implements ChangementColSupprListener
{
	private TableauArbreModel sonTableModel=null;
	private ChangementColSupprListener sonCCSL=null;
	
	
	public Tableau(){}
	
	public Tableau(ListeStockage telContenu, boolean telBoolPremierIsRemovable)
	{
		super();
		sonTableModel=new TableauArbreModel(telContenu, telBoolPremierIsRemovable);
		
		//on applique le model sur le tableau
		super.setModel(sonTableModel);
		
		//ajout de l ecouteur
		sonTableModel.addChangementColSupprListener(this);
		
		
		//Fixe la taille de la colonne suppr
		TableColumn t=this.getColumn(Constantes.NOM_COLS_TABLEAU[Constantes.SUPPR]);
		t.setMaxWidth(Constantes.LARGEUR_COLONNE_SUPPR);
		t.setMinWidth(Constantes.LARGEUR_COLONNE_SUPPR);
		t.setResizable(false);
		
		//Fixe la taille prefere de la colonne nom
		t=this.getColumn(Constantes.NOM_COLS_TABLEAU[Constantes.NOM]);
		t.setPreferredWidth(Constantes.LARGEUR_PREF_COLONNE_NOM);

		
		//On ne peut pas selectionner les lignes
		this.setCellSelectionEnabled(false);
	}
	
	public void setNouveauContenuTableau(ListeStockage telContenu)
	{
		sonTableModel.setContenu(telContenu);
		this.revalidate();
	}
	
	public void ajouteEltTableau(Stockage telElt)
	{
		sonTableModel.ajouteEltTableau(telElt);
		this.revalidate();
	}
	
	public boolean retireEltTableau(Stockage telElt)
	{
		boolean ok=false;
		
		if(getNbLigne()>Constantes.NB_ELT_MIN)
		{
			ok=sonTableModel.retireEltTableau(telElt);
			this.revalidate();
		}
		
		return ok;
	}
	
	public boolean retireEltTableau(int telRang)
	{
		boolean ok=false;
		
		if(getNbLigne()>Constantes.NB_ELT_MIN)
		{
			ok=sonTableModel.retireEltTableau(telRang);
			this.revalidate();
		}
		
		return ok;
	}
	
	public int getNbLigne()
	{
		return sonTableModel.getNbLigne();
	}
	
	public boolean isSelected(int telleL)
	{
		return sonTableModel.isSelected(telleL);
	}
	
	public void selectAll()
	{
		sonTableModel.selectAll();
		this.repaint();
	}
	
	public void deselectAll()
	{
		sonTableModel.deselectAll();
		this.repaint();
	}
	
	public void revalidate()
	{
		if(sonTableModel!=null)
		{
			//on resynchronise les donnees
			sonTableModel.revalidate();
		}
		
		//on revalidate
		super.revalidate();
	}
	
	public void changementColSupprTable(int telleColonne)
	{
		if(sonCCSL!=null)
			sonCCSL.changementColSupprTable(telleColonne);
	}
	
	public void addChangementColSupprListener(ChangementColSupprListener telCCSL)
	{
		sonCCSL=telCCSL;
	}
	
	public boolean aucuneCaseSupprSelectionnee()
	{
		return sonTableModel.aucuneCaseSupprSelectionnee();
	}
	
	public void synchronizeBoolSupprAvecContenu()
	{
		sonTableModel.synchronizeBoolSupprAvecContenu();
	}
	
	public boolean premierEltIsRemovable()
	{
		return sonTableModel.premierEltIsRemovable();
	}
	
	/*public void afficheContenu()
	{
		sonTableModel.afficheContenu();
	}*/
}