package modulecreation;

import java.awt.*;
import javax.swing.JPanel;

import structuresgraphique.UnArbreGraphique;

/**
 * Cette classe reprsente un panel gnrique dans lequel
 * peuvent tre facilement dessins des arbres binaires parfaits.
 * Pour spcialiser cette classe (pour la cration  la souris
 * par exemple), on pourra cre une nouvelle classe hritant de
 * celle ci.
 */
public abstract class UnPanelArbre extends JPanel
{
	protected double sonXTemp=0;
	protected double sonYTemp=0;
	protected UnArbreGraphique sonArbre=null;
	
	/**
	 * Constructeur
	 * @param telArbre l'arbre  afficher dans le panel
	 */
	protected UnPanelArbre(UnArbreGraphique telArbre)
	{	
		sonArbre=telArbre;
		setBackground(ConstantesCreation.COULEUR_FOND);
	}
	
	/**
	 * Dessine les composants
	 * @param g le contexte graphique
	 */
	public void paintComponent(Graphics g)
	{
	  	super.paintComponent(g);
		Graphics2D g2d = (Graphics2D)g;
		
		// Pour le zoom
		g2d.scale(ConstantesCreation.SCALE,ConstantesCreation.SCALE);
		
		// Activation de l'antialisating 
		if(ConstantesCreation.ANTI_ALIAS)
			g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
		else
			g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_OFF);			
			
		// Dessine l'arbre	
		if(sonArbre !=null)	
			sonArbre.dessineArbre(g2d);
	}

	/**
	 * Pour eviter que l'ecran soit effac  chaque repaint()
	 * @param g le contexte graphique
	 */
	public void update(Graphics g) 
	{	
		paint(g);
	}
	
	/**
	 * Retourne l'arbre associ au panel
	 * @return l'arbre
	 */
	public UnArbreGraphique getSonArbre()
	{
		return sonArbre;	
	}
	

}


