package parametres;

import java.awt.Color;

/**
 * Cette classe regroupe les paramtres concernants l'onglet <b>Cration</b>.
 * Elle doit tre instancie dans la classe de dessin(ou autre) de cet onglet
 * et permet de rcuprer en une seule fois les donnes ncessaires,  l'aide de la
 * mthode getParamCreation() de la classe abstraite Parametrage.
 *
 */
 
public class ParametresCreation {
	
	private Color couleurRacine;
	private Color couleurSommets;
	private Color couleurSelection;
	private Color couleurFeuilles;
	private Color couleurTexte;
	
	private double largeurSommets;
	private boolean antiAliasing;
	
	/**
	 * Constructeur initialisant tous les champs
	 *
	 * @param p1 Couleur de la racine
	 * @param p2 Couleur des sommets
	 * @param p3 Couleur de slection
	 * @param p4 Couleur des feuilles
	 * @param p5 Couleur du texte informatif
	 * @param p6 Diamtre des sommets
	 * @param p7 Activation/Dsactivation de l'anti-aliasing
	 */
	public ParametresCreation(Color p1, Color p2, Color p3, Color p4, Color p5, double p6, boolean p7) {
		
		couleurRacine = p1;
		couleurSommets = p2;
		couleurSelection = p3;
		couleurFeuilles = p4;
		couleurTexte = p5;
		largeurSommets = p6;
		antiAliasing = p7;		
	}
	
	/**
	 * Pour obtenir l'tat de l'option d'anti-aliasing.
	 *
	 * @return Un boolen indiquant si l'option est active ou pas.
	 */
	public boolean getAntiAliasing() {
		
		return antiAliasing;
	}
	
	/**
	 * Pour obtenir la couleur des feuilles.
	 *
	 * @return Couleur des feuilles.
	 */
	public Color getCoulFeuilles() {
		
		return couleurFeuilles;
	}
	
	/**
	 * Pour obtenir la couleur de la racine.
	 *
	 * @return Couleur de la racine.
	 */
	public Color getCoulRacine() {
		
		return couleurRacine;
	}
	
	/**
	 * Pour obtenir la couleur des sommets.
	 *
	 * @return Couleur des sommets.
	 */
	public Color getCoulSommets() {
		
		return couleurSommets;
	}
	
	/**
	 * Pour obtenir la couleur des sommets slectionns dans les panels o sont dessins les arbres.
	 *
	 * @return La couleur des sommets slectionns.
	 */
	public Color getCoulSelection() {
		
		return couleurSelection;
	}
	
	/**
	 * Pour obtenir la couleur du texte informatif des sommets.
	 *
	 * @return La couleur du texte.
	 */
	public Color getCoulTexte() {
		
		return couleurTexte;
	}
	
	/**
	 * Pour obtenir la largeur d'un sommet.
	 *
	 * @return Largeur d'un sommet.
	 */
	public double getDiametreSommet() {
		
		return largeurSommets;
	}
}
 
