package parametres;

import java.awt.Color;

/**
 * Cette classe regroupe les paramtres concernants l'onglet <b>Excution</b>.
 * Elle doit tre instancie dans la classe de dessin(ou autre) de cet onglet
 * et permet de rcuprer en une seule fois les donnes ncessaires,  l'aide de la
 * mthode getParamExecution() de la classe abstraite Parametrage.
 *
 */
 
public class ParametresExecution {
	
	private Color couleurRacine;
	private Color couleurSommets;
	private Color couleurSelection;
	private Color couleurFeuilles;
	private Color couleurTexte;
	private Color couleurSuivi;
	private Color couleurCourbe;
	
	private double largeurSommets;
	private int vitesseThrd;
	
	/**
	 * Constructeur initialisant tous les champs
	 *
	 * @param p1 Couleur de la racine
	 * @param p2 Couleur des sommets
	 * @param p3 Couleur de slection
	 * @param p4 Couleur des feuilles
	 * @param p5 Couleur du texte informatif
	 * @param p6 Couleur du sommet suivi
	 * @param p7 Couleur de la courbe
	 * @param p8 Diamtre des sommets
	 * @param p9 Vitesse des threads
	 */
	public ParametresExecution(Color p1, Color p2, Color p3, Color p4, Color p5,
	Color p6, Color p7, double p8, int p9) {
		
		couleurRacine = p1;
		couleurSommets = p2;
		couleurSelection = p3;
		couleurFeuilles = p4;
		couleurTexte = p5;
		couleurSuivi = p6;
		couleurCourbe = p7;
		largeurSommets = p8;
		vitesseThrd = p9;		
	}
	
	/**
	 * Pour obtenir la couleur du sommet suivi.
	 *
	 * @return La couleur du sommet suivi.
	 */
	public Color getCoulSuivi() {
		
		return couleurSuivi;
	}
	
	/**
	 * Pour obtenir la couleur de la courbe.
	 *
	 * @return La couleur de la courbe.
	 */
	public Color getCoulCourbe() {
		
		return couleurCourbe;
	}
	
	/**
	 * Pour obtenir la couleur du texte informatif des sommets.
	 *
	 * @return La couleur du texte.
	 */
	public Color getCoulTexte() {
		
		return couleurTexte;
	}
	
	/**
	 * Pour obtenir la vitesse d'excution des threads.
	 *
	 * @return Vitesse des threads.
	 */
	public int getVitesse() {
		
		return vitesseThrd;
	}
	
	/**
	 * Pour obtenir la largeur d'un sommet.
	 *
	 * @return Largeur d'un sommet.
	 */
	public double getDiametreSommet() {
		
		return largeurSommets;
	}
	
	/**
	 * Pour obtenir la couleur des feuilles.
	 *
	 * @return Couleur des feuilles.
	 */
	public Color getCoulFeuilles() {
		
		return couleurFeuilles;
	}
	
	/**
	 * Pour obtenir la couleur de la racine.
	 *
	 * @return Couleur de la racine.
	 */
	public Color getCoulRacine() {
		
		return couleurRacine;
	}
	
	/**
	 * Pour obtenir la couleur des sommets.
	 *
	 * @return Couleur des sommets.
	 */
	public Color getCoulSommets() {
		
		return couleurSommets;
	}	
	
	/**
	 * Pour obtenir la couleur des sommets slectionns dans les panels o sont dessins les arbres.
	 *
	 * @return La couleur des sommets slectionns.
	 */
	public Color getCoulSelection() {
		
		return couleurSelection;
	}
}